@@ -4,9 +4,13 @@ import re
 import math
 from string import Template
 # from datetime import timedelta
-from mycluster import get_timedelta
+from .mycluster import get_timedelta
 # from subprocess import Popen, PIPE, check_output
-from mycluster import get_data
+from .mycluster import get_data
+from .mycluster import load_template
+
+from jinja2 import Environment, FileSystemLoader
+
 
 """
 sacctmgr show cluster
@@ -25,8 +29,13 @@ def name():
 
 
 def accounts():
-    # sacctmgr list assoc user=exjap
-    pass
+    account_list = []
+
+    with os.popen('sacctmgr --noheader list assoc user=`id -un` format=Account') as f:
+        for line in f:
+            account_list.append(line)
+
+    return account_list
 
 
 def queues():
@@ -48,12 +57,12 @@ def available_tasks(queue_id):
     max_tasks = 0
     queue_name = queue_id
     nc = node_config(queue_id)
-    with os.popen('sinfo -sh -p '+queue_name) as f:
+    with os.popen('sinfo -sh -p ' + queue_name) as f:
         line = f.readline()
         new_line = re.sub(' +', ' ', line.strip())
         line = new_line.split(' ')[3]
-        free_tasks = int(line.split('/')[1])*nc['max task']
-        max_tasks = int(line.split('/')[3])*nc['max task']
+        free_tasks = int(line.split('/')[1]) * nc['max task']
+        max_tasks = int(line.split('/')[3]) * nc['max task']
 
     return {'available': free_tasks, 'max tasks': max_tasks}
 
@@ -61,7 +70,7 @@ def available_tasks(queue_id):
 def tasks_per_node(queue_id):
     queue_name = queue_id
     tasks = 0
-    with os.popen('sinfo -Nelh -p '+queue_name) as f:
+    with os.popen('sinfo -Nelh -p ' + queue_name) as f:
         line = f.readline()
         new_line = re.sub(' +', ' ', line.strip())
         tasks = int(new_line.split(' ')[4])
@@ -73,7 +82,7 @@ def node_config(queue_id):
     queue_name = queue_id
     tasks = 0
     config = {}
-    with os.popen('sinfo -Nelh -p '+queue_name) as f:
+    with os.popen('sinfo -Nelh -p ' + queue_name) as f:
         line = f.readline()
         if len(line):
             new_line = re.sub(' +', ' ', line.strip())
@@ -83,7 +92,8 @@ def node_config(queue_id):
             config['max thread'] = tasks
             config['max memory'] = memory
         else:
-            raise StandardError("Requested partition %s has no nodes" % queue_name)
+            raise Exception(
+                "Requested partition %s has no nodes" % queue_name)
 
     return config
 
@@ -108,205 +118,92 @@ def create_submit(queue_id, **kwargs):
     else:
         raise ValueError("num_threads_per_task must be specified")
 
-    my_name = "myclusterjob"
-    if 'my_name' in kwargs:
-        my_name = kwargs['my_name']
-    my_output = "myclusterjob.out"
-    if 'my_output' in kwargs:
-        my_output = kwargs['my_output']
-    if 'my_script' not in kwargs:
-        pass
-    my_script = kwargs['my_script']
+    my_name = kwargs.get('my_name', "myclusterjob")
+    my_output = kwargs.get('my_output', "myclusterjob.out")
+    my_script = kwargs.get('my_script', None)
+
     if 'mycluster-' in my_script:
         my_script = get_data(my_script)
 
-    if 'user_email' not in kwargs:
-        pass
-    user_email = kwargs['user_email']
+    user_email = kwargs.get('user_email', None)
+    project_name = kwargs.get('project_name', 'default')
 
-    project_name = 'default'
-    if 'project_name' in kwargs:
-        project_name = kwargs['project_name']
+    wall_clock = kwargs.get('wall_clock', '12:00:00')
+    if ':' not in wall_clock:
+        wall_clock = wall_clock + ':00:00'
 
-    wall_clock = '12:00:00'
-    if 'wall_clock' in kwargs:
-        if ':' not in str(kwargs['wall_clock']):
-            wall_clock = str(kwargs['wall_clock'])+':00:00'
-        else:
-            wall_clock = str(kwargs['wall_clock'])
+    num_nodes = int(math.ceil(float(num_tasks) / float(tpn)))
 
-    num_nodes = int(math.ceil(float(num_tasks)/float(tpn)))
+    num_queue_slots = num_nodes * queue_tpn
 
-    num_queue_slots = num_nodes*queue_tpn
+    no_syscribe = kwargs.get('no_syscribe', False)
 
-    record_job = "True"
-    if 'no_syscribe' in kwargs:
-        record_job = ""
+    record_job = not no_syscribe
 
-    if 'openmpi_args' not in kwargs:
-        openmpi_args = "-bysocket -bind-to-socket"
-    else:
-        openmpi_args =  kwargs['openmpi_args']
+    openmpi_args = kwargs.get('openmpi_args', "-bysocket -bind-to-socket")
 
-    script = Template(r"""#!/bin/bash
-#
-# SLURM job submission script generated by MyCluster
-#
-# Job name
-#SBATCH -J $my_name
-# Send status information to this email address.
-#SBATCH --mail-user=$user_email
-# Send me an e-mail when the job has finished.
-#SBATCH --mail-type=ALL
-# Redirect output stream to this file.
-#SBATCH --output $my_output.%j
-# Which project should be charged
-#SBATCH -A $project_name
-# Partition name
-#SBATCH -p $queue_name
-# Number of nodes
-#SBATCH --nodes $num_nodes
-# Number of tasks
-#SBATCH --ntasks $num_tasks
-# Exclusive node use
-#SBATCH --exclusive
-# Do not requeue job on node failure
-#SBATCH --no-requeue
-# How much wallclock time will be required?
-#SBATCH --time=$wall_clock
+    qos = kwargs.get('qos', None)
 
+    template = load_template('slurm.jinja')
 
-export MYCLUSTER_QUEUE=$queue_name
-export MYCLUSTER_JOB_NAME=$my_name
-export NUM_TASKS=$num_tasks
-export TASKS_PER_NODE=$tpn
-export THREADS_PER_TASK=$num_threads_per_task
-if [ "$${THREADS_PER_TASK}" -eq -1 ]; then
-    CPUS_PER_NODE=`echo $${SLURM_JOB_CPUS_PER_NODE}  | cut -d \( -f 1`
-    export THREADS_PER_TASK=`expr $${CPUS_PER_NODE} / $${TASKS_PER_NODE}`
-fi
-
-export NUM_NODES=$num_nodes
-export JOBID=$$SLURM_JOB_ID
-
-# OpenMP configuration
-export OMP_NUM_THREADS=$$THREADS_PER_TASK
-export OMP_PROC_BIND=true
-export OMP_PLACES=sockets
-
-# OpenMPI
-export OMPI_CMD="mpiexec -n $$NUM_TASKS -npernode $$TASKS_PER_NODE $openmpi_args"
-# OpenMPI 1.10
-export OMPI_NEW_CMD="mpiexec -n $$NUM_TASKS --map-by ppr:1:numa --bind-to numa"
-
-# MVAPICH2
-export MV2_CPU_BINDING_LEVEL=SOCKET
-export MV2_CPU_BINDING_POLICY=scatter
-export MVAPICH_CMD="mpiexec -n $$NUM_TASKS -ppn $$TASKS_PER_NODE -bind-to-socket"
-
-# Intel MPI
-# The following variables define a sensible pinning strategy for Intel MPI tasks -
-# this should be suitable for both pure MPI and hybrid MPI/OpenMP jobs:
-export I_MPI_PIN_DOMAIN=omp:compact # Domains are $$OMP_NUM_THREADS cores in size
-export I_MPI_PIN_ORDER=scatter # Adjacent domains have minimal sharing of caches/sockets
-#export I_MPI_FABRICS=shm:ofa
-#export I_MPI_FABRICS=shm:tmi
-#export TMI_CONFIG=<path_to_impi>/intel64/etc/tmi.conf
-export IMPI_CMD="mpiexec -n $$NUM_TASKS -ppn $$TASKS_PER_NODE"
-
-# Summarise environment
-echo -e "JobID: $$JOBID\n======"
-echo "Time: `date`"
-echo "Running on master node: `hostname`"
-echo "Current directory: `pwd`"
-
-if [ "$$SLURM_JOB_NODELIST" ]; then
-        #! Create a machine file:
-        echo $$SLURM_JOB_NODELIST | uniq > machine.file.$$JOBID
-        echo -e "\nNodes allocated:\n================"
-        echo `cat machine.file.$$JOBID | sed -e 's/\..*$$//g'`
-fi
-
-echo -e "\nnumtasks=$num_tasks, numnodes=$num_nodes, tasks_per_node=$tpn (OMP_NUM_THREADS=$$OMP_NUM_THREADS)"
-
-echo -e "\nExecuting command:\n==================\n$my_script\n"
-
-# Run user script
-. $my_script
-
-# Report on completion
-echo -e "\nJob Complete:\n==================\n"
-if [ $record_job ]; then
-    echo -e "\nRecording hardware setup\n==================\n"
-    mycluster --sysscribe $$JOBID
-    if [ "$$MYCLUSTER_APP_NAME" ]; then
-        mycluster --jobid $$JOBID --appname=$$MYCLUSTER_APP_NAME
-    fi
-    if [ "$$MYCLUSTER_APP_DATA" ]; then
-        mycluster --jobid $$JOBID --appdata=$$MYCLUSTER_APP_DATA
-    fi
-fi
-
-echo -e "Complete========\n"
-""")
-    script_str = script.substitute({'my_name':my_name,
-                                    'my_script':my_script,
-                                    'my_output':my_output,
-                                    'user_email':user_email,
-                                    'queue_name':queue_name,
-                                    'num_queue_slots':num_queue_slots,
-                                    'num_tasks':num_tasks,
-                                    'tpn':tpn,
-                                    'num_threads_per_task':num_threads_per_task,
-                                    'num_nodes':num_nodes,
-                                    'project_name': project_name,
-                                    'wall_clock' : wall_clock,
-                                    'record_job' : record_job,
-                                    'openmpi_args': openmpi_args,
-                                   })
-
+    script_str = template.render(my_name=my_name,
+                                 my_script=my_script,
+                                 my_output=my_output,
+                                 user_email=user_email,
+                                 queue_name=queue_name,
+                                 num_queue_slots=num_queue_slots,
+                                 num_tasks=num_tasks,
+                                 tpn=tpn,
+                                 num_threads_per_task=num_threads_per_task,
+                                 num_nodes=num_nodes,
+                                 project_name=project_name,
+                                 wall_clock=wall_clock,
+                                 record_job=record_job,
+                                 openmpi_args=openmpi_args,
+                                 qos=qos)
     return script_str
 
 
-def submit(script_name, immediate, depends_on = None, depends_on_always_run = False):
+def submit(script_name, immediate, depends_on=None,
+           depends_on_always_run=False):
     job_id = None
     if not immediate:
         if depends_on and depends_on_always_run:
-            with os.popen('sbatch --dependency=afterany:%s %s' % (depends_on, script_name)) as f:
+            with os.popen('sbatch --kill-on-invalid-dep=yes --dependency=afterany:%s %s' % (depends_on, script_name)) as f:
                 output = f.readline()
                 try:
                     job_id = int(output.split(' ')[-1].strip())
                 except:
-                    print 'Job submission failed: '+output   
+                    print 'Job submission failed: ' + output
         elif depends_on is not None:
-            with os.popen('sbatch --dependency=afterok:%s %s' % (depends_on, script_name)) as f:
+            with os.popen('sbatch --kill-on-invalid-dep=yes --dependency=afterok:%s %s' % (depends_on, script_name)) as f:
                 output = f.readline()
                 try:
                     job_id = int(output.split(' ')[-1].strip())
                 except:
-                    print 'Job submission failed: '+output   
+                    print 'Job submission failed: ' + output
         else:
-            with os.popen('sbatch '+script_name) as f:
+            with os.popen('sbatch ' + script_name) as f:
                 output = f.readline()
                 try:
                     job_id = int(output.split(' ')[-1].strip())
                 except:
-                    print 'Job submission failed: '+output
+                    print 'Job submission failed: ' + output
                 # Get job id and record in database
     else:
-        with os.popen('grep -- "SBATCH -p" ' + script_name
-                      + ' | sed \'s/#SBATCH//\'') as f:
+        with os.popen('grep -- "SBATCH -p" ' + script_name + ' | sed \'s/#SBATCH//\'') as f:
             partition = f.readline().rstrip()
-        with os.popen('grep -- "SBATCH --nodes" '+script_name+' | sed \'s/#SBATCH//\'') as f:
+        with os.popen('grep -- "SBATCH --nodes" ' + script_name + ' | sed \'s/#SBATCH//\'') as f:
             nnodes = f.readline().rstrip()
-        with os.popen('grep -- "SBATCH --ntasks" '+script_name+' | sed \'s/#SBATCH//\'') as f:
+        with os.popen('grep -- "SBATCH --ntasks" ' + script_name + ' | sed \'s/#SBATCH//\'') as f:
             ntasks = f.readline().rstrip()
-        with os.popen('grep -- "SBATCH -A" '+script_name+' | sed \'s/#SBATCH//\'') as f:
+        with os.popen('grep -- "SBATCH -A" ' + script_name + ' | sed \'s/#SBATCH//\'') as f:
             project = f.readline().rstrip()
-        with os.popen('grep -- "SBATCH -J" '+script_name+' | sed \'s/#SBATCH//\'') as f:
+        with os.popen('grep -- "SBATCH -J" ' + script_name + ' | sed \'s/#SBATCH//\'') as f:
             job = f.readline().rstrip()
 
-        cmd_line = 'salloc --exclusive '+nnodes+' '+partition+' '+ntasks+' '+project+' '+job+' bash ./'+script_name
+        cmd_line = 'salloc --exclusive ' + nnodes + ' ' + partition + ' ' + \
+            ntasks + ' ' + project + ' ' + job + ' bash ./' + script_name
         print cmd_line
 
         import subprocess
@@ -331,7 +228,7 @@ def status():
     status_dict = {}
     with os.popen('squeue -u `whoami`') as f:
         try:
-            f.readline() # read header
+            f.readline()  # read header
             for line in f:
                 new_line = re.sub(' +', ' ', line.strip())
                 job_id = int(new_line.split(' ')[0])
@@ -348,7 +245,7 @@ def status():
 
 def job_stats(job_id):
     stats_dict = {}
-    with os.popen('sacct --noheader --format JobId,Elapsed,TotalCPU,Partition,NTasks,AveRSS,State,ExitCode -P -j '+str(job_id)) as f:
+    with os.popen('sacct --noheader --format JobId,Elapsed,TotalCPU,Partition,NTasks,AveRSS,State,ExitCode -P -j ' + str(job_id)) as f:
         try:
             line = f.readline()
             first_line = line.split('|')
@@ -383,7 +280,8 @@ def job_stats(job_id):
             elif next_line:
                 stats_dict['exit_code'] = int(next_line[7].split(':')[0])
 
-            # stats_dict['mem'] = 0 #float(new_line.split(' ')[4])*int(new_line.split(' ')[3])
+            # stats_dict['mem'] = 0 #float(new_line.split('
+            # ')[4])*int(new_line.split(' ')[3])
         except:
             print('SLURM: Error reading job stats')
 
@@ -404,7 +302,7 @@ def job_stats_enhanced(job_id):
     Get full job and step stats for job_id
     """
     stats_dict = {}
-    with os.popen('sacct --noheader --format JobId,Elapsed,TotalCPU,Partition,NTasks,AveRSS,State,ExitCode,start,end -P -j '+str(job_id)) as f:
+    with os.popen('sacct --noheader --format JobId,Elapsed,TotalCPU,Partition,NTasks,AveRSS,State,ExitCode,start,end -P -j ' + str(job_id)) as f:
         try:
             line = f.readline()
             if line in ["SLURM accounting storage is disabled",
@@ -447,7 +345,7 @@ def job_stats_enhanced(job_id):
                 except:
                     print('SLURM: Error reading job stats')
                     stats_dict['status'] = 'UNKNOWN'
-    with os.popen('squeue --format %%S -h -j '+str(job_id)) as f:
+    with os.popen('squeue --format %%S -h -j ' + str(job_id)) as f:
         try:
             line = f.readline()
             if len(line) > 0:
@@ -458,12 +356,13 @@ def job_stats_enhanced(job_id):
             print('SLURM: Error getting start time')
     return stats_dict
 
+
 def is_in_queue(job_id):
     with os.popen('squeue -j %s' % job_id) as f:
         try:
-            f.readline() # read header
+            f.readline()  # read header
             for line in f:
-                new_line = re.sub(' +',' ',line.strip())
+                new_line = re.sub(' +', ' ', line.strip())
                 q_id = int(new_line.split(' ')[0])
                 if q_id == job_id:
                     return True
@@ -474,7 +373,7 @@ def is_in_queue(job_id):
 
 def running_stats(job_id):
     stats_dict = {}
-    with os.popen('sacct --noheader --format Elapsed -j '+str(job_id)) as f:
+    with os.popen('sacct --noheader --format Elapsed -j ' + str(job_id)) as f:
         try:
             line = f.readline()
             new_line = re.sub(' +', ' ', line.strip())
@@ -482,25 +381,14 @@ def running_stats(job_id):
         except:
             pass
 
-    with os.popen('sstat --noheader --format AveCPU,AveRSS,NTasks -j '
-                  + str(job_id)) as f:
+    with os.popen('sstat --noheader --format AveCPU,AveRSS,NTasks -j ' + str(job_id)) as f:
         try:
             line = f.readline()
             new_line = re.sub(' +', ' ', line.strip())
             ntasks = int(new_line.split(' ')[2])
-            stats_dict['mem'] = (float(new_line.split(' ')[1].replace('K', ''))
-                                 * ntasks)
+            stats_dict['mem'] = (float(new_line.split(' ')[1].replace('K', '')) * ntasks)
             stats_dict['cpu'] = '-'  # float(new_line.split(' ')[0])*ntasks
         except:
             pass
 
     return stats_dict
-
-
-
-
-
-
-
-
-
